
#include <Wire.h>
#include "MP1243.h"

static byte MP1243_VALUES [5] = {0x3C, 0x3C, 0xF6, 0xF6, 0xCE};


MP1243::MP1243()
{

}
//------------------------------------------------------------------------------
// PUBLIC FUNCTIONS
//------------------------------------------------------------------------------
// void MP1243::Init ()
// 
void MP1243::Init ()
{
   Wire.beginTransmission(MP1243_ADDRESS);
   Wire.write (MP1243_VOLL);
   Wire.write (MP1243_VALUES [0]);
   Wire.write (MP1243_ADDRESS);
   Wire.write (MP1243_VOLR);
   Wire.write (MP1243_VALUES [1]);
   Wire.write (MP1243_ADDRESS);
   Wire.write (MP1243_BASS);
   Wire.write (MP1243_VALUES [2]);
   Wire.write (MP1243_ADDRESS);
   Wire.write (MP1243_TREBLE);
   Wire.write (MP1243_VALUES [3]);
   Wire.write (MP1243_ADDRESS);
   Wire.write (MP1243_SWITCH);
   Wire.write (MP1243_VALUES [4]);
   Wire.endTransmission();
}
//------------------------------------------------------------------------------
// void MP1243::SetParam (byte Param, byte Value)
//  
//  :
//           byte Param - ,   
//                     : MP1243_VOLL, MP1243_VOLR,
//                                          MP1243_BASS, MP1243_TREBLE
//           byte Value -  
void MP1243::SetParam (byte Param, byte Value)
{
   if ((Param == MP1243_VOLL) || (Param == MP1243_VOLR))
   {
      if (Value <= VOLUME_MASK)
      {
         MP1243_VALUES [Param] &= ~VOLUME_MASK;
         MP1243_VALUES [Param] |= Value;
      }
   }
   
   else if ((Param == MP1243_BASS) || (Param == MP1243_TREBLE))
   {
      if (Value <= BASS_MASK)
      {
         MP1243_VALUES [Param] &= ~BASS_MASK;
         MP1243_VALUES [Param] |= Value;
      }
   }
   
   Write (Param, MP1243_VALUES [Param]);
}
//------------------------------------------------------------------------------
// void MP1243::SelectInput (byte Input)
//  
//  :
//           byte Input - ,   
//            : MP1243_CHAN_1_STEREO, MP1243_CHAN_2_STEREO,
//           MP1243_CHAN_1_A, MP1243_CHAN_1_B, MP1243_CHAN_2_A, MP1243_CHAN_2_B
void MP1243::SelectInput (byte Input)
{
   MP1243_VALUES [4] &= ~SOURCE_MASK;
   MP1243_VALUES [4] |= Input;
   
   Write (MP1243_SWITCH, MP1243_VALUES [4]);
}
//------------------------------------------------------------------------------
// void MP1243::SelectSignal (byte Signal)
//   
//  :
//           byte Signal -   
//            : MP1243_MONO, MP1243_LINEAR_STEREO,
//                                 MP1243_PSEUDO_STEREO, MP1243_SPATIAL_STEREO
void MP1243::SelectSignal (byte Signal)
{   
   MP1243_VALUES [4] &= ~SIGNAL_MASK;   
   MP1243_VALUES [4] |= Signal;
         
   Write (MP1243_SWITCH, MP1243_VALUES [4]);
}
//------------------------------------------------------------------------------
// void MP1243::Mute_ON ()
//   (  )
void MP1243::Mute_ON ()
{
   MP1243_VALUES [4] |= MP1243_MU_BIT;
   
   Write (MP1243_SWITCH, MP1243_VALUES [4]);
}
//------------------------------------------------------------------------------
// void MP1243::Mute_OFF ()
//   (  )
void MP1243::Mute_OFF ()
{
   MP1243_VALUES [4] &= ~MP1243_MU_BIT;
   
   Write (MP1243_SWITCH, MP1243_VALUES [4]);
}
//------------------------------------------------------------------------------
// byte MP1243::GetParam (byte Param)
//    
//  :
//           byte Param - ,    
//                     : MP1243_VOLL, MP1243_VOLR,
//                                          MP1243_BASS, MP1243_TREBLE
//  :
//              
byte MP1243::GetParam (byte Param)
{
   if ((Param == MP1243_VOLL) || (Param == MP1243_VOLR))
     return (MP1243_VALUES [Param] & VOLUME_MASK);
            
   else if ((Param == MP1243_BASS) || (Param == MP1243_TREBLE))
     return (MP1243_VALUES [Param] & BASS_MASK);
}
//------------------------------------------------------------------------------
// byte MP1243::GetInput ()
//   
//  :   
//            : MP1243_CHAN_1_STEREO, MP1243_CHAN_2_STEREO,
//           MP1243_CHAN_1_A, MP1243_CHAN_1_B, MP1243_CHAN_2_A, MP1243_CHAN_2_B
byte MP1243::GetInput ()
{
   return (MP1243_VALUES [4] & SOURCE_MASK);
}
//------------------------------------------------------------------------------
// byte MP1243::GetSignal ()
//    
//  :   
//            : MP1243_MONO, MP1243_LINEAR_STEREO,
//                                 MP1243_PSEUDO_STEREO, MP1243_SPATIAL_STEREO
byte MP1243::GetSignal ()
{
   return (MP1243_VALUES [4] & SIGNAL_MASK);
}
//------------------------------------------------------------------------------
// boolean MP1243::GetMuteState ()
//    MUTE
//  :   MUTE
// 0 -  MUTE  ( )
// 1 -  MUTE  ( )
boolean MP1243::GetMuteState ()
{
   return ((MP1243_VALUES [4] & MP1243_MU_BIT) >> 5);
}
//------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//------------------------------------------------------------------------------
void MP1243::Write (byte Param, byte Value)
{
   Wire.beginTransmission(MP1243_ADDRESS);
   Wire.write (Param);
   Wire.write (Value);
   Wire.endTransmission();
}


MP1243 MP1243_EQUALIZER = MP1243 ();
