#include <MP1243.h>
#include <MP1090.h>
#include <Wire.h>
#include <PinChangeInt.h>

const int Radio_PWR_Pin = 13;

const int MP1093_VOLL_A = 14;
const int MP1093_VOLL_B = 15;
const int MP1093_VOLR_A = 16;
const int MP1093_VOLR_B = 17;
const int MP1093_BASS_A = 2;
const int MP1093_BASS_B = 3;
const int MP1093_TREBLE_A = 4;
const int MP1093_TREBLE_B = 5;
const int MP1093_BUTTON_1 = 6;
const int MP1093_BUTTON_2 = 7;

const int MP1093_CounterClockWiseRot = 0;
const int MP1093_ClockWiseRot = 1;
const int MP1093_Unchanged = 2;

volatile byte VOLL_FLAG = MP1093_Unchanged;
volatile byte VOLR_FLAG = MP1093_Unchanged;
volatile byte BASS_FLAG = MP1093_Unchanged;
volatile byte TREBLE_FLAG = MP1093_Unchanged;
volatile byte MP1093_BUTTON_1_FLAG = MP1093_Unchanged;
volatile byte MP1093_BUTTON_2_FLAG = MP1093_Unchanged;

void setup()
{
  Serial.begin (9600);
  Wire.begin();
  
  PCintPort::attachInterrupt(MP1093_VOLL_A, &MP1093_VOLL_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOLL_B, &MP1093_VOLL_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOLR_A, &MP1093_VOLR_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOLR_B, &MP1093_VOLR_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BASS_A, &MP1093_BASS_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BASS_B, &MP1093_BASS_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TREBLE_A, &MP1093_TREBLE_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TREBLE_B, &MP1093_TREBLE_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BUTTON_1, &MP1093_BUTTON_1_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BUTTON_2, &MP1093_BUTTON_2_ISR, RISING);
      
  MP1090::Init(Radio_PWR_Pin);
  MP1090::SetStation(95200);
  MP1090::SetVolume(1);
  
  MP1243::Init ();
  MP1243::SelectInput (MP1243_CHAN_2_STEREO);
  MP1243::SelectSignal (MP1243_LINEAR_STEREO);
}
//--------------------------------------------------------
void loop ()
{ 
  if ((VOLL_FLAG == MP1093_ClockWiseRot) || (VOLL_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeVolume (MP1243_VOLL, VOLL_FLAG);
    Serial.print ("Volume Left   ");
    Serial.print (MP1243::GetParam (MP1243_VOLL), HEX);
    Serial.print ("\n");
    VOLL_FLAG = MP1093_Unchanged;
  }
   
  if ((VOLR_FLAG == MP1093_ClockWiseRot) || (VOLR_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeVolume (MP1243_VOLR, VOLR_FLAG);
    Serial.print ("Volume Right   ");
    Serial.print (MP1243::GetParam (MP1243_VOLR), HEX);
    Serial.print ("\n");
    VOLR_FLAG = MP1093_Unchanged;
  }
    
  if ((BASS_FLAG == MP1093_ClockWiseRot) || (BASS_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBassTreble (MP1243_BASS, BASS_FLAG);
    Serial.print ("BASS   ");
    Serial.print (MP1243::GetParam (MP1243_BASS), HEX);
    Serial.print ("\n");
    BASS_FLAG = MP1093_Unchanged;
  }
  
  if ((TREBLE_FLAG == MP1093_ClockWiseRot) || (TREBLE_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBassTreble (MP1243_TREBLE, TREBLE_FLAG);
    Serial.print ("TREBLE   ");
    Serial.print (MP1243::GetParam (MP1243_TREBLE), HEX);
    Serial.print ("\n");
    TREBLE_FLAG = MP1093_Unchanged;
  }
  
  if (MP1093_BUTTON_1_FLAG == MP1093_ClockWiseRot)
  {
    ChangeInput ();
    Serial.print ("Input   ");
    Serial.print (MP1243::GetInput (), HEX);
    Serial.print ("\n");
    MP1093_BUTTON_1_FLAG = MP1093_Unchanged;
  }
  
  if (MP1093_BUTTON_2_FLAG == MP1093_ClockWiseRot)
  {
    ChangeSignal ();
    Serial.print ("Signal   ");
    Serial.print (MP1243::GetSignal (), HEX);
    Serial.print ("\n");
    MP1093_BUTTON_2_FLAG = MP1093_Unchanged;
  }
}
//--------------------------------------------------------
void MP1093_VOLL_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOLL_A)
  {
   delay (100);
   
   if ((digitalRead (MP1093_VOLL_A)) && (!digitalRead (MP1093_VOLL_B)))
   {
     VOLL_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_VOLL_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOLL_B)
  {
    delay (100);
    if ((digitalRead (MP1093_VOLL_B)) && (!digitalRead (MP1093_VOLL_A)))
    {
      VOLL_FLAG = MP1093_CounterClockWiseRot;
    }
  }
}
//-----------------------------------------------------------------
void MP1093_VOLR_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOLR_A)
  {
     delay (100);
     if ((digitalRead (MP1093_VOLR_A)) && (!digitalRead (MP1093_VOLR_B)))
     {
        VOLR_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_VOLR_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOLR_B)
  {
   delay (100);
   if ((digitalRead (MP1093_VOLR_B)) && (!digitalRead (MP1093_VOLR_A)))
   {
     VOLR_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BASS_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BASS_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BASS_A)) && (!digitalRead (MP1093_BASS_B)))
     {
        BASS_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BASS_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BASS_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BASS_B)) && (!digitalRead (MP1093_BASS_A)))
   {
     BASS_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_TREBLE_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TREBLE_A)
  {
     delay (100);
     if ((digitalRead (MP1093_TREBLE_A)) && (!digitalRead (MP1093_TREBLE_B)))
     {
        TREBLE_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_TREBLE_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TREBLE_B)
  {
   delay (100);
   if ((digitalRead (MP1093_TREBLE_B)) && (!digitalRead (MP1093_TREBLE_A)))
   {
     TREBLE_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BUTTON_1_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BUTTON_1)
  {
   delay (100);
   if ((digitalRead (MP1093_BUTTON_1)))
   {
     MP1093_BUTTON_1_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BUTTON_2_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BUTTON_2)
  {
   delay (100);
   if ((digitalRead (MP1093_BUTTON_2)))
   {
     MP1093_BUTTON_2_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void ChangeVolume (byte Volume, byte VOLUME_FLAG)
{ 
  byte CurValue = MP1243::GetParam (Volume);
  byte NewValue = CurValue;
  
  if (VOLUME_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue < 0x3F)
      NewValue++;
  }
  
  else if (VOLUME_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue > 0x1B)
     NewValue--;
  }
  
  if (NewValue != CurValue)
    MP1243::SetParam (Volume, NewValue);
}
//-----------------------------------------------------------------
void ChangeBassTreble (byte Filter, byte FILTER_FLAG)
{
  byte CurValue = MP1243::GetParam (Filter);
  byte NewValue = CurValue;

  if (FILTER_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue < 0x0B)
      NewValue++;
  }
  
  else if (FILTER_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue > 0x02)
      NewValue--;
  }
  
  if (NewValue != CurValue)
    MP1243::SetParam (Filter, NewValue);
}
//-----------------------------------------------------------------
void ChangeInput ()
{
  byte CurValue = MP1243::GetInput ();
  byte NewValue = CurValue;

  if (NewValue < 0x07)
    NewValue++;
  
  else if (NewValue == 0x07)
    NewValue = 0x02;
  
  if (NewValue != CurValue)
    MP1243::SelectInput (NewValue);
}
//-----------------------------------------------------------------
void ChangeSignal ()
{
  byte CurValue = MP1243::GetSignal()>>3;
  byte NewValue = CurValue;

  if (NewValue < 0x03)
    NewValue++;
  
  else if (NewValue == 0x03)
    NewValue = 0x00;
  
  if (NewValue != CurValue)
    MP1243::SelectSignal (NewValue<<3);
}
//-----------------------------------------------------------------
